//*****************************************************************************
//
// Microsoft Windows Media
// Copyright (C) Microsoft Corporation. All rights reserved.
//
// FileName:            ProfileObject.cpp
//
// Abstract:            The implementation for the profile object helper
//                      classes.  These classes are used as the item data in
//                      the dialog controls, and contain the configuration of
//                      profile objects.
//
//*****************************************************************************

#include "stdafx.h"
//#include "GenProfile.h"
#include "ProfileObject.h"
#include "Macros.h"
#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/*
** Profile object
*/

///////////////////////////////////////////////////////////////////////////////
CProfileObject::CProfileObject()
{
    m_cRef = 1;
}

///////////////////////////////////////////////////////////////////////////////
CProfileObject::~CProfileObject()
{
}

///////////////////////////////////////////////////////////////////////////////
ProfileObjectType CProfileObject::Type()
{
    return OT_Unknown;
}

///////////////////////////////////////////////////////////////////////////////
ULONG CProfileObject::AddRef()
{
    return ++m_cRef;
}

///////////////////////////////////////////////////////////////////////////////
ULONG CProfileObject::Release()
{
    if ( 0 == --m_cRef )
    {
        delete this;
        return 0;
    }

    return m_cRef;
}


/*
** Stream object
*/

///////////////////////////////////////////////////////////////////////////////
CStream::CStream()
{
    m_strName = "";
    m_stStreamType = ST_Arbitrary;
    m_dwStreamFormatIndex = 0;
    m_dwFormatStringIndex = 0;
    m_dwBufferWindow = 3000;
    m_dwVideoWidth = 320;
    m_dwVideoHeight = 200;
    m_dwSecondsPerKeyframe = 3000;
    m_dwFPS = 30;
    m_dwVideoQuality = 100;
    m_dwStreamBitrate = 100000;
    m_guidArbitraryType = GUID_NULL;
    m_bVideoIsVBR = FALSE;
    m_dwVideoMaxBufferWindow = 0;
    m_dwVideoMaxBitrate = 10000;
    m_nVideoVBRMode = VBR_QUALITYBASED;
    m_bHasSMPTE = FALSE;
    m_wStreamNumber = 0;
    m_dwStreamCodecIndex = 0;
    m_dwVideoVBRQuality = 0;
}


///////////////////////////////////////////////////////////////////////////////
CStream::~CStream()
{
    PrepareForDeletion();
}


///////////////////////////////////////////////////////////////////////////////
ProfileObjectType CStream::Type()
{
    return OT_Stream;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::PrepareForDeletion()
{
    int nIndex;

    for ( nIndex = 0; nIndex < m_dependentObjects.GetSize(); nIndex++ )
    {
        assert( m_dependentObjects[nIndex] );
        m_dependentObjects[nIndex]->NotifyObjectDeleted( this );
        SAFE_RELEASE( m_dependentObjects[nIndex] );
    }
    m_dependentObjects.RemoveAll();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStream::AddDependent( CProfileObject* pDependentObject )
{
    if ( !pDependentObject )
    {
        return E_INVALIDARG;
    }

    SAFE_ADDREF( pDependentObject );
    m_dependentObjects.Add( pDependentObject );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStream::RemoveDependent( CProfileObject* pDependentObject )
{
    BOOL fFound;
    INT nIndex;

    if ( !pDependentObject )
    {
        return E_INVALIDARG;
    }

    fFound = false;
    for( nIndex = 0; nIndex < m_dependentObjects.GetSize(); nIndex++ )
    {
        if ( m_dependentObjects[nIndex] == pDependentObject )
        {
            fFound = true;
            break;
        }
    }

    if ( fFound )
    {
        m_dependentObjects.RemoveAt( nIndex );
        SAFE_RELEASE( pDependentObject );
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
ULONG CStream::DependentCount()
{
    return m_dependentObjects.GetSize();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStream::GetDependent( INT nIndex, CProfileObject **ppProfileObject )
{
    if ( !ppProfileObject )
    {
        return E_POINTER;
    }
    if ( nIndex < 0 || nIndex >= m_dependentObjects.GetSize() )
    {
        return E_INVALIDARG;
    }

    *ppProfileObject = m_dependentObjects[ nIndex ];
    SAFE_ADDREF( (*ppProfileObject) );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStream::NotifyObjectDeleted( CProfileObject* pDeletedObject )
{
    int nIndex;

    if ( !pDeletedObject )
    {
        return E_INVALIDARG;
    }

    for ( nIndex = 0; nIndex < m_dependentObjects.GetSize(); nIndex++ )
    {
        if ( m_dependentObjects[nIndex] == pDeletedObject )
        {
            m_dependentObjects.RemoveAt( nIndex );
            SAFE_RELEASE( pDeletedObject );
            break;
        }
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStream::SetName( CString strName )
{
    m_strName = strName;

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
CString CStream::GetName()
{
    return m_strName;
}


///////////////////////////////////////////////////////////////////////////////
StreamType CStream::GetStreamType( )
{
    return m_stStreamType;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamType( StreamType stNewType )
{
    assert( ( stNewType == ST_Audio ) || 
            ( stNewType == ST_Video ) || 
            ( stNewType == ST_Script ) || 
            ( stNewType == ST_Image ) || 
            ( stNewType == ST_Web ) || 
            ( stNewType == ST_File ) || 
            ( stNewType == ST_Arbitrary ) );

    m_stStreamType = stNewType;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamFormatIndex( DWORD dwIndex )
{
    m_dwStreamFormatIndex = dwIndex;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetStreamFormatIndex()
{
    return m_dwStreamFormatIndex;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamFormatStringIndex( DWORD dwFormatStringIndex )
{
    m_dwFormatStringIndex = dwFormatStringIndex;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetStreamFormatStringIndex()
{
    return m_dwFormatStringIndex;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamBufferWindow( DWORD dwBufferWindow )
{
    m_dwBufferWindow = dwBufferWindow;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetStreamBufferWindow()
{
    return m_dwBufferWindow;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoHeight( DWORD dwHeight )
{
    m_dwVideoHeight = dwHeight;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoHeight()
{
    return m_dwVideoHeight;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoWidth( DWORD dwVideoWidth )
{
    m_dwVideoWidth = dwVideoWidth;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoWidth()
{
    return m_dwVideoWidth;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamBitrate( DWORD dwBitrate )
{
    m_dwStreamBitrate = dwBitrate;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetStreamBitrate()
{
    return m_dwStreamBitrate;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoFPS( DWORD dwFPS )
{
    m_dwFPS = dwFPS;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoFPS()
{
    return m_dwFPS;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoSecondsPerKeyframe()
{
    return m_dwSecondsPerKeyframe;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoSecondsPerKeyframe( DWORD dwSecondsPerKeyframe )
{
    m_dwSecondsPerKeyframe = dwSecondsPerKeyframe;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoQuality( DWORD dwVideoQuality )
{
    assert( dwVideoQuality <= 100 );

    m_dwVideoQuality = dwVideoQuality;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoQuality()
{
    return m_dwVideoQuality;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetArbitraryType( GUID guidType )
{
    m_guidArbitraryType = guidType;
}


///////////////////////////////////////////////////////////////////////////////
GUID CStream::GetArbitraryType()
{
    return m_guidArbitraryType;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoIsVBR( BOOL fIsVBR )
{
    m_bVideoIsVBR = fIsVBR;
}


///////////////////////////////////////////////////////////////////////////////
BOOL CStream::GetVideoIsVBR()
{
    return m_bVideoIsVBR;
}


///////////////////////////////////////////////////////////////////////////////
VIDEO_VBR_MODE CStream::GetVideoVBRMode()
{
    return m_nVideoVBRMode;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoVBRMode( VIDEO_VBR_MODE nVBRMode )
{
    assert( ( nVBRMode == VBR_OFF ) ||
            ( nVBRMode == VBR_QUALITYBASED ) ||
            ( nVBRMode == VBR_CONSTRAINED ) ||
            ( nVBRMode == VBR_UNCONSTRAINED ) );

    m_nVideoVBRMode = nVBRMode;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoMaxBitrate()
{
    return m_dwVideoMaxBitrate;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoMaxBitrate( DWORD dwMaxBitrate )
{
    m_dwVideoMaxBitrate = dwMaxBitrate;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoMaxBufferWindow()
{
    return m_dwVideoMaxBufferWindow;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoMaxBufferWindow( DWORD dwMaxBufferWindow )
{
    m_dwVideoMaxBufferWindow = dwMaxBufferWindow;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetIsSMPTE( BOOL fIsSMPTE )
{
    m_bHasSMPTE = fIsSMPTE;
}


///////////////////////////////////////////////////////////////////////////////
BOOL CStream::GetIsSMPTE()
{
    return m_bHasSMPTE;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamNumber( WORD wStreamNumber )
{
    m_wStreamNumber = wStreamNumber;
}


///////////////////////////////////////////////////////////////////////////////
WORD CStream::GetStreamNumber()
{
    return m_wStreamNumber;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetStreamCodecIndex( DWORD dwStreamCodecIndex )
{
    m_dwStreamCodecIndex = dwStreamCodecIndex;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetStreamCodecIndex()
{
    return m_dwStreamCodecIndex;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CStream::GetVideoVBRQuality()
{
    return m_dwVideoVBRQuality;
}


///////////////////////////////////////////////////////////////////////////////
void CStream::SetVideoVBRQuality( DWORD dwVBRQuality )
{
    assert( dwVBRQuality <= 100 );

    m_dwVideoVBRQuality = dwVBRQuality;
}


/*
** Mutex object
*/

///////////////////////////////////////////////////////////////////////////////
CMutex::CMutex()
{
    m_mtType = MT_Bitrate;
}


///////////////////////////////////////////////////////////////////////////////
CMutex::~CMutex()
{
    PrepareForDeletion();
}


///////////////////////////////////////////////////////////////////////////////
ProfileObjectType CMutex::Type()
{
    return OT_Mutex;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CMutex::NotifyObjectDeleted( CProfileObject* pDeletedObject )
{
    int nIndex;

    if ( !pDeletedObject )
    {
        return E_INVALIDARG;
    }

    for ( nIndex = 0; nIndex < m_streamsInMutex.GetSize(); nIndex++ )
    {
        if ( m_streamsInMutex[nIndex] == pDeletedObject )
        {
            m_streamsInMutex.RemoveAt( nIndex );
            SAFE_RELEASE( pDeletedObject );
            break;
        }
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
void CMutex::PrepareForDeletion()
{
    int nIndex;

    for ( nIndex = 0; nIndex < m_streamsInMutex.GetSize(); nIndex++ )
    {
        assert( m_streamsInMutex[nIndex] );
        m_streamsInMutex[nIndex]->NotifyObjectDeleted( this );
        SAFE_RELEASE( m_streamsInMutex[nIndex] );
    }
    m_streamsInMutex.RemoveAll();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CMutex::AddStream( CStream* pNewStream )
{
    if ( !pNewStream )
    {
        return E_INVALIDARG;
    }

    SAFE_ADDREF( pNewStream );
    m_streamsInMutex.Add( (CProfileObject*) pNewStream );
    pNewStream->AddDependent( this );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CMutex::RemoveStream( CStream* pStream )
{
    BOOL fFound;
    INT nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    fFound = false;
    for( nIndex = 0; nIndex < m_streamsInMutex.GetSize(); nIndex++ )
    {
        if ( m_streamsInMutex[nIndex] == pStream )
        {
            fFound = true;
            break;
        }
    }

    if ( fFound )
    {
        m_streamsInMutex.RemoveAt( nIndex );
        pStream->RemoveDependent( this );
        SAFE_RELEASE( pStream );
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
BOOL CMutex::InObject( CStream* pStream )
{
    BOOL fFound;
    INT nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    fFound = false;
    for( nIndex = 0; nIndex < m_streamsInMutex.GetSize(); nIndex++ )
    {
        if ( m_streamsInMutex[nIndex] == pStream )
        {
            fFound = true;
            break;
        }
    }

    return fFound;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CMutex::RemoveAllStreams()
{
    HRESULT hr = S_OK;

    do
    {
        while ( m_streamsInMutex.GetSize() > 0 )
        {
            hr = RemoveStream( (CStream*) m_streamsInMutex[0] );
            if ( FAILED( hr ) )
            {
                break;
            }
        }
    }
    while ( FALSE );

    return hr;
}


///////////////////////////////////////////////////////////////////////////////
void CMutex::SetMutexType( MUTEX_TYPE mtType )
{
    assert( ( mtType == MT_Bitrate ) ||
         ( mtType == MT_Language ) ||
         ( mtType == MT_Presentation ) );

    m_mtType = mtType;
}


///////////////////////////////////////////////////////////////////////////////
MUTEX_TYPE CMutex::GetMutexType()
{
    return m_mtType;
}


///////////////////////////////////////////////////////////////////////////////
INT CMutex::StreamCount()
{
    return m_streamsInMutex.GetSize();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CMutex::GetStream( INT nStreamIndex, CStream** ppStream )
{
    if ( !ppStream )
    {
        return E_POINTER;
    }
    if ( ( nStreamIndex < 0) || ( nStreamIndex >= m_streamsInMutex.GetSize() ) )
    {
        return E_INVALIDARG;
    }

    *ppStream = (CStream*) m_streamsInMutex[nStreamIndex];
    SAFE_ADDREF( (*ppStream) );

    return S_OK;
}


/*
** Stream prioritization object
*/

///////////////////////////////////////////////////////////////////////////////
CStreamPrioritizationObject::CStreamPrioritizationObject()
{
}


///////////////////////////////////////////////////////////////////////////////
CStreamPrioritizationObject::~CStreamPrioritizationObject()
{
    PrepareForDeletion();
}


///////////////////////////////////////////////////////////////////////////////
ProfileObjectType CStreamPrioritizationObject::Type()
{
    return OT_StreamPrioritization;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::NotifyObjectDeleted( CProfileObject* pDeletedObject )
{
    int nIndex;

    if ( !pDeletedObject )
    {
        return E_INVALIDARG;
    }

    for ( nIndex = 0; nIndex < m_streamPriorities.GetSize(); nIndex++ )
    {
        if ( m_streamPriorities[nIndex] == pDeletedObject )
        {
            m_streamPriorities.RemoveAt( nIndex );
            SAFE_RELEASE( pDeletedObject );
            break;
        }
    }

    for ( nIndex = 0; nIndex < m_mandatoryStreams.GetSize(); nIndex++ )
    {
        if ( m_mandatoryStreams[nIndex] == pDeletedObject )
        {
            m_mandatoryStreams.RemoveAt( nIndex );
            SAFE_RELEASE( pDeletedObject );
            break;
        }
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
void CStreamPrioritizationObject::PrepareForDeletion()
{
    int nIndex;

    for ( nIndex = 0; nIndex < m_streamPriorities.GetSize(); nIndex++ )
    {
        assert( m_streamPriorities[nIndex] );
        m_streamPriorities[nIndex]->NotifyObjectDeleted( this );
        SAFE_RELEASE( m_streamPriorities[nIndex] );
    }
    m_streamPriorities.RemoveAll();

    for ( nIndex = 0; nIndex < m_mandatoryStreams.GetSize(); nIndex++ )
    {
        assert( m_mandatoryStreams[nIndex] );
        m_mandatoryStreams[nIndex]->NotifyObjectDeleted( this );
        SAFE_RELEASE( m_mandatoryStreams[nIndex] );
    }
    m_mandatoryStreams.RemoveAll();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::AddStream( CStream* pNewStream )
{
    if ( !pNewStream )
    {
        return E_INVALIDARG;
    }

    SAFE_ADDREF( pNewStream );
    m_streamPriorities.Add( pNewStream );
    pNewStream->AddDependent( this );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::SetStreamMandatory( CStream* pStream, BOOL fIsMandatory )
{
    BOOL fFound;
    INT nFoundIndex;
    int nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    //
    // Search the list for the stream
    //
    fFound = false;
    nFoundIndex = -1;
    for ( nIndex = 0; nIndex < m_mandatoryStreams.GetSize(); nIndex++ )
    {
        if ( m_mandatoryStreams[ nIndex ] == pStream )
        {
            fFound = true;
            nFoundIndex = nIndex;
            break;
        }
    }

    if ( fFound )
    {
        if ( !fIsMandatory )
        {
            SAFE_RELEASE( pStream );
            m_mandatoryStreams.RemoveAt( nFoundIndex );
        }
    }
    else
    {
        if ( fIsMandatory )
        {
            SAFE_ADDREF( pStream );
            m_mandatoryStreams.Add( pStream );
        }
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::ClearMandatoryStreams()
{
    HRESULT hr = S_OK;

    while ( m_mandatoryStreams.GetSize() > 0 )
    {
        hr = SetStreamMandatory( (CStream*) m_mandatoryStreams[0], FALSE );
        if ( FAILED( hr ) )
        {
            break;
        }
    }

    return hr;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::GetStreamMandatory( CStream* pStream, BOOL *pbIsMandatory )
{
    BOOL fFound;
    int nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }
    if ( !pbIsMandatory )
    {
        return E_POINTER;
    }

    //
    // Search the list for the stream
    //
    fFound = false;
    for ( nIndex = 0; nIndex < m_mandatoryStreams.GetSize(); nIndex++ )
    {
        if ( m_mandatoryStreams[ nIndex ] == pStream )
        {
            fFound = true;
            break;
        }
    }

    *pbIsMandatory = fFound;

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::IncreasePriority( CStream* pStream )
{
    BOOL fFound;
    INT nFoundIndex;
    int nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    //
    // Search the list for the stream
    //
    fFound = false;
    nFoundIndex = -1;
    for ( nIndex = 0; nIndex < m_streamPriorities.GetSize(); nIndex++ )
    {
        if ( m_streamPriorities[ nIndex ] == pStream )
        {
            fFound = true;
            nFoundIndex = nIndex;
            break;
        }
    }

    if ( fFound )
    {
        if ( nFoundIndex > 0 )
        {
            m_streamPriorities.RemoveAt( nFoundIndex );
            m_streamPriorities.InsertAt( nFoundIndex - 1, pStream );
        }

        return S_OK;
    }

    return E_FAIL;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::DecreasePriority( CStream* pStream )
{
    BOOL fFound;
    INT nFoundIndex;
    int nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    //
    // Search the list for the stream
    //
    fFound = false;
    nFoundIndex = -1;
    for ( nIndex = 0; nIndex < m_streamPriorities.GetSize(); nIndex++ )
    {
        if ( m_streamPriorities[ nIndex ] == pStream )
        {
            fFound = true;
            nFoundIndex = nIndex;
            break;
        }
    }

    if ( fFound )
    {
        if ( nFoundIndex < m_streamPriorities.GetSize() )
        {
            m_streamPriorities.RemoveAt( nFoundIndex );
            m_streamPriorities.InsertAt( nFoundIndex + 1, pStream );
        }

        return S_OK;
    }

    return E_FAIL;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CStreamPrioritizationObject::GetStreamWithPriority( INT nPriorityIndex, CStream** ppStream )
{
    if ( !ppStream )
    {
        return E_POINTER;
    }
    if ( ( nPriorityIndex < 0 ) || ( nPriorityIndex > m_streamPriorities.GetSize() ))
    {
        return E_INVALIDARG;
    }

    *ppStream = (CStream*) m_streamPriorities[ nPriorityIndex ];
    SAFE_ADDREF( (*ppStream) );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
INT CStreamPrioritizationObject::StreamCount()
{
    return m_streamPriorities.GetSize();
}



/*
** Bandwidth sharing object
*/

///////////////////////////////////////////////////////////////////////////////
CBandwidthSharingObject::CBandwidthSharingObject()
{
    m_dwSharedBitrate = 100000;
    m_dwBufferWindow = 3000;
    m_guidBandwidthSharingType = CLSID_WMBandwidthSharing_Exclusive;
}

///////////////////////////////////////////////////////////////////////////////
CBandwidthSharingObject::~CBandwidthSharingObject()
{
    PrepareForDeletion();
}


///////////////////////////////////////////////////////////////////////////////
ProfileObjectType CBandwidthSharingObject::Type()
{
    return OT_BandwidthSharing;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CBandwidthSharingObject::NotifyObjectDeleted( CProfileObject* pDeletedObject )
{
    int nIndex;

    if ( !pDeletedObject )
    {
        return E_INVALIDARG;
    }

    for ( nIndex = 0; nIndex < m_streamsSharingBandwidth.GetSize(); nIndex++ )
    {
        if ( m_streamsSharingBandwidth[nIndex] == pDeletedObject )
        {
            m_streamsSharingBandwidth.RemoveAt( nIndex );
            SAFE_RELEASE( pDeletedObject );
            break;
        }
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
void CBandwidthSharingObject::PrepareForDeletion()
{
    int nIndex;

    for ( nIndex = 0; nIndex < m_streamsSharingBandwidth.GetSize(); nIndex++ )
    {
        assert( m_streamsSharingBandwidth[nIndex] );
        m_streamsSharingBandwidth[nIndex]->NotifyObjectDeleted( this );
        SAFE_RELEASE( m_streamsSharingBandwidth[nIndex] );
    }
    m_streamsSharingBandwidth.RemoveAll();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CBandwidthSharingObject::AddStream( CStream* pNewStream )
{
    if ( !pNewStream )
    {
        return E_INVALIDARG;
    }

    SAFE_ADDREF( pNewStream );
    m_streamsSharingBandwidth.Add( pNewStream );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CBandwidthSharingObject::RemoveStream( CStream* pStream )
{
    BOOL fFound;
    INT nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    fFound = false;
    for( nIndex = 0; nIndex < m_streamsSharingBandwidth.GetSize(); nIndex++ )
    {
        if ( m_streamsSharingBandwidth[nIndex] == pStream )
        {
            fFound = true;
            break;
        }
    }

    if ( fFound )
    {
        m_streamsSharingBandwidth.RemoveAt( nIndex );
        SAFE_RELEASE( pStream );
    }

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
BOOL CBandwidthSharingObject::InObject( CStream* pStream )
{
    BOOL fFound;
    INT nIndex;

    if ( !pStream )
    {
        return E_INVALIDARG;
    }

    fFound = false;
    for( nIndex = 0; nIndex < m_streamsSharingBandwidth.GetSize(); nIndex++ )
    {
        if ( m_streamsSharingBandwidth[nIndex] == pStream )
        {
            fFound = true;
            break;
        }
    }

    return fFound;
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CBandwidthSharingObject::RemoveAllStreams()
{
    HRESULT hr = S_OK;

    do
    {
        while ( m_streamsSharingBandwidth.GetSize() > 0 )
        {
            hr = RemoveStream( (CStream*) m_streamsSharingBandwidth[0] );
            if ( FAILED( hr ) )
            {
                break;
            }
        }
    }
    while ( FALSE );

    return hr;
}


///////////////////////////////////////////////////////////////////////////////
void CBandwidthSharingObject::SetSharedBitrate( DWORD dwBitrate )
{
    m_dwSharedBitrate = dwBitrate;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CBandwidthSharingObject::GetSharedBitrate( )
{
    return m_dwSharedBitrate;
}


///////////////////////////////////////////////////////////////////////////////
INT CBandwidthSharingObject::StreamCount()
{
    return m_streamsSharingBandwidth.GetSize();
}


///////////////////////////////////////////////////////////////////////////////
HRESULT CBandwidthSharingObject::GetStream( INT nStreamIndex, CStream** ppStream )
{
    if ( !ppStream )
    {
        return E_POINTER;
    }
    if ( ( nStreamIndex < 0) || ( nStreamIndex >= m_streamsSharingBandwidth.GetSize() ) )
    {
        return E_INVALIDARG;
    }

    *ppStream = (CStream*) m_streamsSharingBandwidth[nStreamIndex];
    SAFE_ADDREF( (*ppStream) );

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
void CBandwidthSharingObject::SetBandwidthSharingType( GUID guidType )
{
    m_guidBandwidthSharingType = guidType;
}


///////////////////////////////////////////////////////////////////////////////
GUID CBandwidthSharingObject::GetBandwidthSharingType()
{
    return m_guidBandwidthSharingType;
}


///////////////////////////////////////////////////////////////////////////////
DWORD CBandwidthSharingObject::GetBufferWindow()
{
    return m_dwBufferWindow;
}


///////////////////////////////////////////////////////////////////////////////
void CBandwidthSharingObject::SetBufferWindow( DWORD dwBufferWindow )
{
    m_dwBufferWindow = dwBufferWindow;
}
